/*
 * 
 *  This file is part of XmlParser.
 *
 *  Foobar is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Foobar is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package org.sourceprojects.xmlparser;

import org.sourceprojects.xmlparser.exceptions.XmlParserInitializationException;

/**
 * Defines a factory API that enables applications to obtain instances of
 * {@link org.sourceprojects.xmlparser.ResourceResolver} either by given
 * standard implmentation or by configuring own implementation.
 * 
 * @author noctarius
 * @since 0.0.1
 */
public final class ResourceResolverFactory {
	/**
	 * Classname for the standard
	 * {@link org.sourceprojects.xmlparser.ResourceResolver} class to be created
	 * if no special implementation is given
	 */
	public static final String STANDARD_RESOURCERESOLVER_CLASS = "org.sourceprojects.xmlparser.internal.dom.ResourceResolverImpl";

	/**
	 * Obtain a new instance of a
	 * {@link org.sourceprojects.xmlparser.ResourceResolver}<br>
	 * Implementation used for
	 * {@link org.sourceprojects.xmlparser.ResourceResolver} is the standard
	 * one. The classloader to load up this implementation is the actual
	 * ContextClassLoader of the actual thread.
	 * 
	 * @return New instance of standard ResourceResolver implementation
	 * @throws XmlParserInitializationException
	 *             If some exception occurs on initialization of the new
	 *             instance the exception is wrapped into a
	 *             {@link XmlParserInitializationException}
	 */
	public static final ResourceResolver newInstance()
			throws XmlParserInitializationException {

		ClassLoader classLoader = Thread.currentThread()
				.getContextClassLoader();
		return newInstance(STANDARD_RESOURCERESOLVER_CLASS, classLoader);
	}

	/**
	 * Obtain a new instance of a
	 * {@link org.sourceprojects.xmlparser.ResourceResolver}<br>
	 * Implementation used for
	 * {@link org.sourceprojects.xmlparser.ResourceResolver} is the standard
	 * one. The classloader to load up those implementations is the given
	 * classloader.
	 * 
	 * @param classLoader
	 *            The classloader to load classes
	 * @return New instance of standard ResourceResolver implementation
	 * @throws XmlParserInitializationException
	 *             If some exception occurs on initialization of the new
	 *             instance the exception is wrapped into a
	 *             {@link XmlParserInitializationException}
	 */
	public static final ResourceResolver newInstance(ClassLoader classLoader)
			throws XmlParserInitializationException {

		return newInstance(STANDARD_RESOURCERESOLVER_CLASS, classLoader);
	}

	/**
	 * Obtain a new instance of a
	 * {@link org.sourceprojects.xmlparser.ResourceResolver}<br>
	 * Implementation used for
	 * {@link org.sourceprojects.xmlparser.ResourceResolver} must be given as
	 * parameter. The classloader to load up those implementations is the given
	 * classloader.
	 * 
	 * @param resolverImplementationClassname
	 * @param classLoader
	 *            The classloader to load classes
	 * @return New instance of standard ResourceResolver implementation
	 * @throws XmlParserInitializationException
	 *             If some exception occurs on initialization of the new
	 *             instance the exception is wrapped into a
	 *             {@link XmlParserInitializationException}
	 */
	@SuppressWarnings("unchecked")
	public static final ResourceResolver newInstance(
			String resolverImplementationClassname, ClassLoader classLoader)
			throws XmlParserInitializationException {

		Class<? extends ResourceResolver> resolverClass;
		try {
			resolverClass = (Class<? extends ResourceResolver>) classLoader
					.loadClass(resolverImplementationClassname);

			return resolverClass.newInstance();
		} catch (Exception e) {
			throw new XmlParserInitializationException(
					"Parser initialization failed", e);
		}

	}
}
