/*
 * 
 *  This file is part of XmlParser.
 *
 *  Foobar is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Foobar is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package org.sourceprojects.xmlparser.internal;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.nio.CharBuffer;
import java.util.List;

import org.sourceprojects.xmlparser.ProgressListener;

/**
 * 
 * @author noctarius
 * @since 0.0.7
 */
public class ProgressAwareReader extends Reader {
	private final Object lock = new Object();
	
	private final List<ProgressListener> listeners;
	private final InputStreamReader reader;
	private final int fileLength;
	private int lastPercent = 0;
	private Integer position = 0;
	private int markedPosition = 0;

	public ProgressAwareReader(InputStream inputStream,
			List<ProgressListener> listeners) throws IOException {
		this.fileLength = inputStream.available();
		this.reader = new InputStreamReader(inputStream);
		this.listeners = listeners;
	}

	@Override
	public void mark(int readAheadLimit) throws IOException {
		if (reader.markSupported()) {
			markedPosition = readAheadLimit;
			reader.mark(readAheadLimit);
		}
	}

	@Override
	public void close() throws IOException {
		reader.close();
	}

	@Override
	public int read(char[] cbuf, int off, int len) throws IOException {
		return reader.read(cbuf, off, len);
	}

	@Override
	public int read() throws IOException {
		notifyListeners(4);
		return reader.read();
	}

	@Override
	public int read(char[] cbuf) throws IOException {
		notifyListeners(cbuf.length);
		return reader.read(cbuf);
	}

	@Override
	public int read(CharBuffer target) throws IOException {
		notifyListeners(target.length());
		return reader.read(target);
	}

	@Override
	public void reset() throws IOException {
		notifyListenersAboutReset();
		reader.reset();
	}

	@Override
	public long skip(long n) throws IOException {
		notifyListeners((int) n);
		return reader.skip(n);
	}

	private void notifyListeners(int bytes) {
		synchronized (lock) {
			position += bytes;

			Double percent = (position * 100.) / fileLength;
			if (percent.intValue() > lastPercent) {
				lastPercent = percent.intValue();

				for (ProgressListener listener : listeners)
					listener.notify(lastPercent);
			}
		}
	}

	private void notifyListenersAboutReset() {
		synchronized (lock) {
			position = markedPosition;

			Double percent = (position * 100.) / fileLength;
			if (percent.intValue() > lastPercent) {
				lastPercent = percent.intValue();

				for (ProgressListener listener : listeners)
					listener.notifyReset();
			}
		}
	}
}
