/*
 * 
 *  This file is part of XmlParser.
 *
 *  Foobar is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Foobar is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package org.sourceprojects.xmlparser.internal;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.logging.Logger;

import javax.xml.XMLConstants;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;

import org.sourceprojects.xmlparser.AbstractTagParser;
import org.sourceprojects.xmlparser.DefinitionBuilder;
import org.sourceprojects.xmlparser.ParserContext;
import org.sourceprojects.xmlparser.ProgressListener;
import org.sourceprojects.xmlparser.ResourceResolver;
import org.sourceprojects.xmlparser.TagDefinitionParser;
import org.sourceprojects.xmlparser.XmlParser;
import org.sourceprojects.xmlparser.exceptions.XmlParserParsingException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * 
 * @author noctarius
 * @since 0.0.1
 * 
 * @param <B>
 */
public class XmlParserImpl<B> implements XmlParser<B> {
	private static final Logger LOGGER = Logger.getLogger(XmlParserImpl.class
			.getName());

	private final TagDefinitionParserContainer<B> tagDefinitionParsers = new TagDefinitionParserContainer<B>();
	private final List<ProgressListener> listeners = new ArrayList<ProgressListener>();
	private ResourceResolver resourceResolver;
	private ErrorHandler errorHandler;
	private ParserContext<B> parserContext;
	private boolean validatingSchema = true;

	@Override
	public void addTagDefinitionParser(
			TagDefinitionParser<B> tagDefinitionParser) {
		addTagDefinitionParserInternal(tagDefinitionParser);
	}

	@Override
	public void addTagDefinitionParsers(
			TagDefinitionParser<B>[] tagDefinitionParsers) {
		for (TagDefinitionParser<B> tagDefinitionParser : tagDefinitionParsers)
			addTagDefinitionParserInternal(tagDefinitionParser);
	}

	@Override
	public void removeTagDefinitionParser(
			TagDefinitionParser<B> tagDefinitionParser) {
		removeTagDefinitionParserInternal(tagDefinitionParser);
	}

	@Override
	public void removeTagDefinitionParsers(
			TagDefinitionParser<B>[] tagDefinitionParsers) {
		for (TagDefinitionParser<B> tagDefinitionParser : tagDefinitionParsers)
			removeTagDefinitionParserInternal(tagDefinitionParser);
	}

	@Override
	public void parse(InputStream inputStream) throws XmlParserParsingException {
		try {
			Document document = createReader(inputStream);
			DefinitionBuilder definitionBuilder = new DefinitionBuilderImpl(
					document);

			for (Node node = document.getFirstChild(); node != null; node = node
					.getNextSibling())
				readNode(node, definitionBuilder);
		} catch (Exception e) {
			throw new XmlParserParsingException("Parsing failed", e);
		}
	}

	@Override
	public void parse(Reader reader) throws XmlParserParsingException {
		try {
			parse(new ReaderInputStream(reader));
		} catch (IOException e) {
			throw new XmlParserParsingException("Parsing failed", e);
		}
	}

	@Override
	public void parse(String xml) throws XmlParserParsingException {
		parse(new StringReader(xml));
	}

	@Override
	public void parse(File file) throws XmlParserParsingException {
		try {
			parse(new FileReader(file));
		} catch (FileNotFoundException e) {
			throw new XmlParserParsingException("Parsing failed", e);
		}
	}

	@Override
	public Document transform(InputStream inputStream)
			throws XmlParserParsingException {
		try {
			Document document = createReader(inputStream);
			Document transformedDocument = createDocumentBuilder()
					.newDocument();

			DefinitionBuilder definitionBuilder = new DefinitionBuilderImpl(
					transformedDocument);

			for (Node node = document.getFirstChild(); node != null; node = node
					.getNextSibling()) {
				Node result = transformNode(node, definitionBuilder);

				if (result != null)
					transformedDocument.appendChild(result);
			}

			return transformedDocument;
		} catch (Exception e) {
			throw new XmlParserParsingException("Parsing failed", e);
		}
	}

	@Override
	public Document transform(String xml) throws XmlParserParsingException {
		return transform(new StringReader(xml));
	}

	@Override
	public Document transform(Reader reader) throws XmlParserParsingException {
		try {
			return transform(new ReaderInputStream(reader));
		} catch (IOException e) {
			throw new XmlParserParsingException("Parsing failed", e);
		}
	}

	@Override
	public Document transform(File file) throws XmlParserParsingException {
		try {
			return transform(new FileReader(file));
		} catch (FileNotFoundException e) {
			throw new XmlParserParsingException("Parsing failed", e);
		}
	}

	@Override
	public ResourceResolver getResourceResolver() {
		return resourceResolver;
	}

	@Override
	public void setResourceResolver(ResourceResolver resourceResolver) {
		this.resourceResolver = resourceResolver;
	}

	@Override
	public ErrorHandler getErrorHandler() {
		return errorHandler;
	}

	@Override
	public void setErrorHandler(ErrorHandler errorHandler) {
		this.errorHandler = errorHandler;
	}

	@Override
	public ParserContext<B> getParserContext() {
		return parserContext;
	}

	@Override
	public void setParserContext(ParserContext<B> parserContext) {
		this.parserContext = parserContext;
	}

	private Document createReader(InputStream inputStream) throws Exception {
		DocumentBuilder builder = createDocumentBuilder();
		builder.setEntityResolver(new EntityResolverImpl(resourceResolver));
		Document document = builder.parse(new InputSource(
				new ProgressAwareReader(inputStream, listeners)));

		return document;
	}

	private DocumentBuilder createDocumentBuilder() throws Exception {
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

		// setting options for clean parsing
		factory.setIgnoringComments(true);
		factory.setIgnoringElementContentWhitespace(true);
		factory.setNamespaceAware(true);
		if (isValidatingSchema()) {
			try {
				factory.setSchema(createSchema());
			} catch (UnsupportedOperationException e) {
				LOGGER.warning("Warning: Schema isn't supported");
				// ignore since it isn't supported by all implementations
			}
		}

		return factory.newDocumentBuilder();
	}

	private void readNode(Node node, DefinitionBuilder definitionBuilder)
			throws XmlParserParsingException {
		if (node.getNodeType() == Node.ELEMENT_NODE) {
			String namespaceURI = node.getNamespaceURI();
			String tagname = node.getLocalName();

			TagDefinitionParser<B> parser = findParserByNamespaceAndTag(
					namespaceURI, tagname, (Element) node);

			if (parser != null) {
				QName qName = new QName(namespaceURI, tagname);
				parser.parseElement((Element) node, tagname, qName,
						parserContext, definitionBuilder);
			}
		}

		if (node.hasChildNodes())
			for (Node child = node.getFirstChild(); child != null; child = child
					.getNextSibling())
				readNode(child, definitionBuilder);
	}

	private Node transformNode(Node node, DefinitionBuilder definitionBuilder)
			throws XmlParserParsingException {
		Node n;

		String namespaceURI = node.getNamespaceURI();
		String tagname = node.getLocalName();

		TagDefinitionParser<B> parser = null;
		if (namespaceURI != null)
			parser = findParserByNamespaceAndTag(namespaceURI, tagname,
					(Element) node);

		if (parser != null) {
			QName qName = new QName(namespaceURI, tagname);
			n = parser.transformElement((Element) node, tagname, qName,
					parserContext, definitionBuilder);
		} else {
			n = definitionBuilder.getDocument().importNode(node, false);
		}

		if (node.hasChildNodes()) {
			if (n != null) {
				for (Node child = node.getFirstChild(); child != null; child = child
						.getNextSibling()) {

					Node result = transformNode(child, definitionBuilder);

					if (result != null)
						n.appendChild(result);
				}
			}
		}

		return n;
	}

	private Schema createSchema() throws SAXException {
		SchemaFactory factory = SchemaFactory
				.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
		factory.setResourceResolver(resourceResolver);
		factory.setErrorHandler(errorHandler);
		return factory.newSchema();
	}

	private void addTagDefinitionParserInternal(
			TagDefinitionParser<B> tagDefinitionParser) {
		String namespaceURI = tagDefinitionParser.getNamespace();

		if (resourceResolver == null)
			throw new NullPointerException("ResourceResolver is null");

		if (resourceResolver.resolveResource(null, namespaceURI, null, null,
				null) == null)
			throw new NullPointerException("Namespace " + namespaceURI
					+ " cannot be resolved");

		if (!(tagDefinitionParser instanceof AbstractTagParser<?>))
			throw new RuntimeException(
					"TagParser to add is no class extending AbstractTagParser");

		Set<TagDefinitionParser<B>> parsers = tagDefinitionParsers
				.get(namespaceURI);
		if (parsers == null)
			parsers = new HashSet<TagDefinitionParser<B>>();

		parsers.add(tagDefinitionParser);

		tagDefinitionParsers.put(namespaceURI, parsers);
	}

	private void removeTagDefinitionParserInternal(
			TagDefinitionParser<B> tagDefinitionParser) {
		String namespaceURI = tagDefinitionParser.getNamespace();

		Set<TagDefinitionParser<B>> parsers = tagDefinitionParsers
				.get(namespaceURI);

		if (parsers != null) {
			parsers.remove(tagDefinitionParser);
			tagDefinitionParsers.put(namespaceURI, parsers);
		}
	}

	private TagDefinitionParser<B> findParserByNamespaceAndTag(
			String namespaceURI, String tagname, Element element) {
		TagDefinitionParser<B> defaultParser = null;

		Set<TagDefinitionParser<B>> parsers = tagDefinitionParsers
				.get(namespaceURI);

		if (parsers == null)
			throw new NullPointerException(
					"No namespace parsers found for namespace " + namespaceURI);

		for (TagDefinitionParser<B> parser : parsers)
			for (String tag : parser.getTags())
				if (tag.contains("\\")) {
					QName[] qNames = generateParentQName(tag, namespaceURI);
					boolean found = true;

					for (int i = qNames.length - 1; i > -1; i--) {
						QName qName = qNames[i];
						String localPart = qName.getLocalPart();
						String namespacePart = qName.getNamespaceURI();

						if (!localPart.equals(element.getLocalName())
								|| !namespacePart.equals(element
										.getNamespaceURI())) {
							found = false;
							break;
						}

						if (i > 0)
							if (element.getParentNode() == null) {
								found = false;
								break;
							} else {
								element = (Element) element.getParentNode();
							}
					}

					if (found)
						return parser;
				} else if (tag.equals(tagname)) {
					return parser;
				} else if (tag.equals("*")) {
					defaultParser = parser;
				}

		return defaultParser;
	}

	private QName[] generateParentQName(String tag, String namespaceURI) {
		String[] tags = tag.split("\\\\");

		List<QName> qNames = new ArrayList<QName>();
		for (String t : tags) {
			if (t.contains("{")) {
				String pNamespaceURI = t.substring(t.indexOf("{") + 1, t
						.indexOf("}"));
				String pTag = t.substring(t.indexOf("}") + 1);

				qNames.add(new QName(pNamespaceURI, pTag));
			} else {
				qNames.add(new QName(namespaceURI, t));
			}
		}

		return qNames.toArray(new QName[qNames.size()]);
	}

	@Override
	public boolean isValidatingSchema() {
		return validatingSchema;
	}

	@Override
	public void setValidatingSchema(boolean validatingSchema) {
		this.validatingSchema = validatingSchema;
	}

	@Override
	public void addProgressListener(ProgressListener listener) {
		listeners.add(listener);
	}

	@Override
	public void removeProgressListener(ProgressListener listener) {
		listeners.remove(listener);
	}
}
