/*
 * 
 *  This file is part of XmlParser.
 *
 *  Foobar is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Foobar is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package org.sourceprojects.xmlparser.internal.dom;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.logging.Logger;

import org.sourceprojects.xmlparser.NamespaceResolver;
import org.sourceprojects.xmlparser.ResourceResolver;
import org.sourceprojects.xmlparser.exceptions.XmlParserInitializationException;
import org.sourceprojects.xmlparser.internal.resolver.BasicNamespaceResolver;
import org.sourceprojects.xmlparser.internal.resolver.BasicNamespaceResolver.NamespaceResolverHandlerContainer;
import org.w3c.dom.ls.LSInput;

/**
 * 
 * @author noctarius
 * @since 0.0.1
 */
public class ResourceResolverImpl implements ResourceResolver {
	private static final Logger LOGGER = Logger.getLogger(ResourceResolverImpl.class.getName());
	
	private final Map<String, NamespaceResolver> namespaceResolvers = new HashMap<String, NamespaceResolver>();
	private final BasicNamespaceResolver basicNamespaceResolver;
	private boolean showWarning = true;


	public ResourceResolverImpl() {
		try {
			final List<NamespaceResolverHandlerContainer> containers = new ArrayList<NamespaceResolverHandlerContainer>();

			final ClassLoader cl = Thread.currentThread()
					.getContextClassLoader();

			final Enumeration<URL> resources = cl
					.getResources("META-INF/xmlparser/schema.properties");

			while (resources.hasMoreElements()) {
				URL url = resources.nextElement();

				LOGGER.info("Found schema.properties in '" + url.toString()
						+ "'");

				final Properties properties = new Properties();
				properties.load(url.openStream());

				final Enumeration<Object> keys = properties.keys();
				while (keys.hasMoreElements()) {
					final String key = (String) keys.nextElement();
					final String value = properties.getProperty(key);

					containers.add(new NamespaceResolverHandlerContainer(key,
							value));
				}
			}

			basicNamespaceResolver = new BasicNamespaceResolver(containers
					.toArray(new NamespaceResolverHandlerContainer[containers
							.size()]), cl);
		} catch (final IOException e) {
			throw new XmlParserInitializationException(
					"Failed internal initialization", e);
		}
	}

	@Override
	public boolean isShowWarning() {
		return showWarning;
	}

	@Override
	public void setShowWarning(boolean showWarning) {
		this.showWarning = showWarning;
	}

	@Override
	public void registerNamespaceResolver(NamespaceResolver namespaceResolver) {
		String namespaceURI = namespaceResolver.getNamespaceURI();

		if (namespaceResolvers.get(namespaceURI) != null && isShowWarning()) {
			StringBuilder sb = new StringBuilder();
			sb.append("Namespace ").append(namespaceURI).append(
					" is already defined and will be overridden.");
			LOGGER.warning(sb.toString());
		}

		namespaceResolvers.put(namespaceURI, namespaceResolver);
	}

	@Override
	public void unregisterNamespaceResolver(NamespaceResolver namespaceResolver) {
		unregisterNamespaceResolver(namespaceResolver.getNamespaceURI());
	}

	@Override
	public void unregisterNamespaceResolver(String namespaceURI) {
		namespaceResolvers.remove(namespaceURI);
	}

	@Override
	public LSInput resolveResource(String type, String namespaceURI,
			String publicId, String systemId, String baseURI) {

		NamespaceResolver resolver = namespaceResolvers.get(namespaceURI);
		if (resolver != null) {
			return resolver.getInput(type, namespaceURI, publicId, systemId,
					baseURI);
		} else {
			return basicNamespaceResolver.getInput(type, namespaceURI,
					publicId, systemId, baseURI);
		}
	}

}
